# Generate the distribution of in-degrees given the maximum in-degree and the total number of edges in a graph.
# maxdeg: maximum in-degree; node: the number of nodes; nedge: the total number of edges
degreeG <- function(maxdeg, node, nedge) {
    degree.dist <- rep(0,maxdeg+1)
    nx <- node; nex <- nedge
    if(maxdeg>=2) {
        for (i in maxdeg:2) {
            lower <- max(0, ceiling(nex-(nx-i/2)*(i-1)))
            upper <- min(nx-i, floor(nex/i))
            if(lower<upper)    {degree.dist[i+1] <- sample(lower:upper,1)}
            else if(lower==upper)    {degree.dist[i+1] <- lower}
            else  {next}
            nx <- nx-degree.dist[i+1]
            nex <- nex-i*degree.dist[i+1]
            if(nx<=0 || nex<0)    stop("Error")
        }
    }
    if (nx>=(nex+1))   degree.dist[2] <- nex
    else  stop("Error!")
    degree.dist[1] <- node-sum(degree.dist[2:(maxdeg+1)])
    return(degree.dist)
}



# Generate DAGs given the maximum in-degree and the total number of edges.
GGen <- function(maxdeg, node, nedge) {

	ordex <- matrix(0, maxdeg, node)		# matrix listing the parents of each node (similar to the adjacency list)
	ts <- sample(1:node)					# topological sort
	degree.dist <- degreeG(maxdeg, node, nedge)
	indegree <- rep(0,node)					# indegree is matched to ts
	ind <- rep(TRUE, node)
	for (i in (maxdeg+1):1) {
		if(degree.dist[i]>0) {
			candidate <- (i:node)[ind[i:node]]
			if(length(candidate)==1)   {
				indegree[candidate] <- (i-1)
				ind[candidate] <- FALSE
			}        
			else  {
				pos <- sample(candidate,degree.dist[i])
				indegree[pos] <- (i-1)
				ind[pos] <- FALSE
			}
		}
	}
	for (i in 1:node) {
		np <- indegree[i]
		if(np==0)   next
		else if(i==2)    ordex[1,ts[i]] <- ts[1]
		else   {
			pa <- sample(ts[1:(i-1)], np)
			ordex[1:np,ts[i]] <- sort(pa)
		}
	}
	trueG <- matrix(0,node,node)
	for (i in 1:node) {
		trueG[ordex[ordex[,i]!=0,i],i] <- 1
	}

	return(list(ordex=ordex, trueG=trueG, ts=ts, degree.dist=degree.dist))

}



# Generate Gaussian data from DAGs using the outputs from the function GGen as the inputs.
# coeff: the beta coefficient; noi: the number of interventions/data blocks; nobs: the number of interventional samples in each data block 
DatGen <- function(ordex, ts, coeff, noi, nobs) {

	maxdeg <- nrow(ordex); node <- ncol(ordex)
	beta <- matrix(coeff, maxdeg, node)
	ivn <- sort(sample(1:node, noi))					# nodes that have interventional data
	fX <- matrix(rnorm(noi*nobs,0,1),ncol=noi)
	data <- .Call("datGenC", as.integer(node),as.integer(maxdeg),as.integer(ordex),as.integer(ts),
                     as.integer(noi),as.integer(nobs),as.integer(ivn),as.double(fX),as.double(beta))
	data <- matrix(data, ncol=node, byrow=TRUE)
	obslist <- list()									# list of indices of observational samples for each node (excluding interventional samples)
	kdex <- 0
	for (k in 1:node) {
		if (any((k-ivn)==0)) {
			kdex <- kdex+1
			obslist[[k]] <- (1:nrow(data))[-((nobs*kdex-nobs+1):(nobs*kdex))]
		} else {
			obslist[[k]] <- 1:nrow(data)
		}
	}
	return(list(data=data, ivn=ivn, obslist=obslist))

}



# Compute solutions to the penalized likelihood problem on a grid of lambdas.
cdpathC <- function(node, data, obslist, eorder, fmlam=0.001, nlam=50, eps=1e-4, beta, lambda, gamma=0.15, lowerbound=1e-4)  {

	argind1 <- missing(lambda); argind2 <- missing(beta)

    obs.leng <- sapply(obslist, length)
    ncoef <- node*node
	sigma <- rep(0,node)
    xlist <- list()
    ylist <- list()
	Normx <- NULL
	inverseWeights <- matrix(0,node,node)
	scale.factor <- NULL
    for (i in 1:node) {
        xtemp <- scale(data[obslist[[i]], ], TRUE, FALSE)
		normx <- sqrt(colSums(xtemp^2))
        xtemp <- scale(xtemp, FALSE, normx)
		scale.factor <- c(scale.factor,normx[i]/normx)
		
		inverseWeights[-i,i] <- W <- pmax(abs(lm(xtemp[,i]~xtemp[,-i]-1)$coef),lowerbound)^gamma		# compute weights for the adaptive lasso
		xtemp[,-i] <- xtemp[,-i]*rep(W,rep(obs.leng[i],node-1))            								# rescale x for the adaptive lasso
		
		Normx <- c(Normx, colSums(xtemp^2))
        ylist[[i]] <- xtemp[,i]
		sigma[i] <- sum(xtemp[,i]^2)/obs.leng[i]
        xtemp[, i] <- 0
        xlist[[i]] <- xtemp
    }
    re.prod <- .Call("prodG",as.integer(node),xlist,ylist,as.integer(obs.leng))
    inprod <- re.prod$inprod                            # inner products of each feature with y initially
    feature.prod <- re.prod$fprod                     	# inner products between features

    if(argind1) {
        maxtmp1 <- max(abs(inprod)/rep(sigma,rep(node,node)))
		max.lam <- 7*maxtmp1
        min.lam <- fmlam*max.lam
        lambda <- exp(seq(log(max.lam), log(min.lam), log(fmlam)/(nlam-1)))
    }
    if(argind2)    beta <- matrix(0, node, node)
   
	
	norm <- penalty <- rep(0,node)
	for (i in 1:node) {                                             
        residual <- ylist[[i]]-xlist[[i]] %*% beta[,i]
        norm[i] <- sum(residual^2)
		penalty[i] <- lambda[1]*sum(abs(beta[,i]))
    }
    rsq <- sum(obs.leng*log(sigma)/2)+sum(norm/(2*sigma))+sum(penalty)
    
	eorleng <- nrow(eorder)
    eorder <- t(eorder)
    result <- .C("CDGridR", as.integer(node), as.double(beta), as.double(lambda), as.double(inprod), as.double(feature.prod),
              as.integer(eorder), as.double(rsq), as.double(eps), coef=double(nlam*ncoef), RSQ=double(nlam), ITER=integer(nlam), as.integer(nlam-1),
			  as.double(sigma), sig=double(nlam*node), as.double(norm), as.integer(obs.leng), as.integer(eorleng))
    coef <- matrix(result$coef*rep(inverseWeights,nlam), ncol=ncoef, byrow=TRUE)
	
	
	
	
	# use new weights to recompute the solutions
	inverseWeights <- matrix(abs(coef[nlam,])^gamma,ncol=node)
    for (i in 1:node) {
        xtemp <- scale(data[obslist[[i]], ], TRUE, FALSE)
		normx <- sqrt(colSums(xtemp^2))
        xtemp <- scale(xtemp, FALSE, normx)
		
		xtemp[,-i] <- xtemp[,-i]*rep(inverseWeights[-i,i],rep(obs.leng[i],node-1))

        ylist[[i]] <- xtemp[,i]
		sigma[i] <- sum(xtemp[,i]^2)/obs.leng[i]
        xtemp[, i] <- 0
        xlist[[i]] <- xtemp
    }
	re.prod <- .Call("prodG",as.integer(node),xlist,ylist,as.integer(obs.leng))
    inprod <- re.prod$inprod
    feature.prod <- re.prod$fprod

	if(argind1) {
        maxtmp1 <- max(abs(inprod)/rep(sigma,rep(node,node)))
		max.lam <- 7*maxtmp1
        min.lam <- fmlam*max.lam
        lambda <- exp(seq(log(max.lam), log(min.lam), log(fmlam)/(nlam-1)))
    }
    if(argind2)    beta <- matrix(0, node, node)
   
	norm <- penalty <- rep(0,node)
	for (i in 1:node) {                                             
        residual <- ylist[[i]]-xlist[[i]] %*% beta[,i]
        norm[i] <- sum(residual^2)
		penalty[i] <- lambda[1]*sum(abs(beta[,i]))
    }
    rsq <- sum(obs.leng*log(sigma)/2)+sum(norm/(2*sigma))+sum(penalty)
    
    result <- .C("CDGridR", as.integer(node), as.double(beta), as.double(lambda), as.double(inprod), as.double(feature.prod),
              as.integer(eorder), as.double(rsq), as.double(eps), coef=double(nlam*ncoef), RSQ=double(nlam), ITER=integer(nlam), as.integer(nlam-1),
			  as.double(sigma), sig=double(nlam*node), as.double(norm), as.integer(obs.leng), as.integer(eorleng))
    coef <- matrix(result$coef*rep(inverseWeights,nlam), ncol=ncoef, byrow=TRUE)
	
	
	
	
	coef <- coef*rep(scale.factor,rep(nlam,ncoef))
    return(list(coef=coef, lambda=lambda))
    
}



# Compute the negative log-likelihood using unpenalized coefficients.
plog <- function(node, G, data, obslist)  {

   obs.leng <- sapply(obslist, length)
   RSS <- rep(0,node)
   for (i in 1:node) {
		pa <- which(G[,i]!=0)
        xtemp <- scale(data[obslist[[i]], ], TRUE, FALSE)
        if(length(pa)>0) {
			RSS[i] <- sum(lm(xtemp[,i]~xtemp[,pa]-1)$res^2)
		} else {
		    RSS[i] <- sum(xtemp[,i]^2)
		}
   }
   pLog <- sum(obs.leng*log(RSS/obs.leng)/2)+sum(obs.leng/2)
   return(pLog)
	
}


