###########################################################################################################################################
# 	Run the following scripts to produce the CD result using one data set from a randomly generated DAG that has p = 50 and beta = 0.5.
#	Run "R CMD BATCH sample.r" in the current folder from the command line.
#
#	Before running the codes, you need to
#	1. Create shared libraries by running 
#   	$ R CMD SHLIB Basic.c
#   	$ R CMD SHLIB CD.c
#	2. Load both shared libraries into R using dyn.load.
#	3. Source R codes in "DAG.r" by using source.
#
#	Note: 	For large networks, if the dimension of the data set is large, the program consumes a fair amount of memory. This problem
#			normally occurs for DAGs with 200 nodes in our experiments, where memory consumption exceeds 2.5GB if the data set has
#			6,000 samples. Thus, our current implementation should not be applied to large data sets generated from DAGs with more than
#			200 nodes.  
###########################################################################################################################################

# source DAG.r. Replace ... with the directory of the corresponding file.
source(".../DAG.r")

# load shared libraries (.so under unix-like or .dll under windows). Replace ... with the directory of the corresponding files.
dyn.load(".../Basic.so")	
dyn.load(".../CD.so")

###########################################################################################################################################
# set all necessary function arguments. See readme for details.

# set arguments for function "GGen". 
node <- 50
maxdeg <- 4
nedge <- 2*node

# set arguments for function "DatGen". The total sample size is determined by (noi*nobs).
coeff <- 0.5
noi <- node
nobs <- 120

# set arguments for function "cdpathC". 
# we usually set "Gamma" to be around 0.15. We occasionally use Gamma = 0.5 when both "node" and "coeff" are large.
# change "fmlam" and "nlam" to obtain a longer sequence of lambda values on a finer grid.
fmlam <- 0.001
nlam <- 50
eps <- 1e-4
Gamma <- 0.15
lowerbound <- 1e-4
eorder <- which(matrix(TRUE, node, node),arr.ind=TRUE)[,c(2,1)]
eorder <- matrix(eorder[which(eorder[,2]>eorder[,1]),], ncol=2)
###########################################################################################################################################






# generate one DAG and save. You can modify the argument "file" to specify the directory for saving the output.
otglist <- GGen(maxdeg, node, nedge)
write.table(otglist$trueG, file="dag.txt", sep=" ", row.names=FALSE, col.names=FALSE)

# generate one interventional data set from the DAG. You can modify the argument "file" to specify the directory for saving the output.
data.list <- DatGen(otglist$ordex, otglist$ts, coeff, noi, nobs)
write.table(data.list$data, file="data.txt", sep=" ", row.names=FALSE, col.names=FALSE)
write.table(t(data.list$ivn), file="ivn.txt", sep=" ", row.names=FALSE, col.names=FALSE)

# run the CD algorithm.
# note: function "cdpathC" runs the CD algorithm twice. See "Section 3 Asymptotic Properties" of the paper for discussion. 
coefficient <- cdpathC(node, data.list$data, data.list$obslist, eorder=eorder, fmlam=fmlam, nlam=nlam, eps=eps, gamma=Gamma, lowerbound=lowerbound)$coef






# create a matrix to store results.
stat <- matrix(0, nrow=nlam, ncol=6)
colnames(stat) <- c("predicted","expected","reversed","missed","fal.pos","logL")

# evaluate the estimated DAGs against the true DAG.
trueG <- otglist$trueG
for (j in 1:nlam) {
    mg <- matrix(0,node,node)
    mg[coefficient[j,]!=0] <-1
	stat[j,1] <- predicted <- length(mg[mg==1])									# compute the number of predicted edges
    stat[j,2] <- expected <- length(mg[(mg==trueG) & (mg==1)])					# compute the number of expected edges
    stat[j,3] <- reversed <- length(mg[(mg-t(trueG)==0) & (mg==1)])				# compute the number of reversed edges
    stat[j,4] <- missed <- length(which((mg==0) & (trueG==1) & (t(mg)==0)))		# compute the number of missed edges
	stat[j,5] <- fal.pos <- predicted-expected-reversed							# compute the number of false positive edges
	stat[j,6] <- -plog(node, mg, data.list$data, data.list$obslist)				# compute the log-likelihood
}

# save results to a file. You can modify the argument "file" to specify the directory for saving the output.
write.table(stat, file="CDstat.txt", sep=" ", row.names=FALSE, col.names=TRUE)


